@@ -1,3 +1,5 @@
+from datetime import datetime
+
 from libcomxml.core import XmlModel, XmlField
 from primestg.order.base import (OrderHeader, CntOrderHeader)
 from primestg.utils import ContractTemplates, DLMSTemplates, datetimetoprime, name2octet, prepare_params
@@ -6,14 +8,84 @@ from pytz import timezone
 
 TZ = timezone('Europe/Madrid')
 
-SUPPORTED_ORDERS = ['B03', 'B04', 'B07', 'B09', 'B11']
+SUPPORTED_ORDERS = ['B02', 'B03', 'B04', 'B07', 'B09', 'B11']
 
-DEFAULT_DC_STG_VERSION = '3.1.c'
 
 def is_supported(order_code):
     return order_code in SUPPORTED_ORDERS
 
 
+# B02 Node classes
+class Contrato1(XmlModel):
+    """
+    The class to instance B04 Contract
+    Parameters:
+        powers: powers (in W) for every 6 periods
+    """
+    def __init__(self, payload):
+        powers = payload.get('powers')
+        self.contrato1 = XmlField(
+            'Contrato1', attributes={
+                'TR1': powers[0],
+                'TR2': powers[1],
+                'TR3': powers[2],
+                'TR4': powers[3],
+                'TR5': powers[4],
+                'TR6': powers[5],
+            }
+        )
+        super(Contrato1, self).__init__('Contrato1', 'contrato1')
+
+
+class B02:
+    """
+    The class used to instance B02 order.
+
+    :return: B02 order with parameters
+    """
+    def __init__(self, generic_values, payload):
+        self.generic_values = generic_values
+        self.order = CntOrderHeader(
+            generic_values.get('id_pet'),
+            generic_values.get('id_req'),
+            generic_values.get('cnc'),
+            generic_values.get('cnt'),
+            generic_values.get('version'),
+        )
+        self.order.cnc.cnt.feed({'payload': B02Payload(payload)})
+        # Load generic order with values
+
+
+class B02Payload(XmlModel):
+    """
+    The class used to instance B02 parameters.
+    Supported parameters:
+        actvation_date: Datetime of activation localized or not. It gets always CE(S)T timezone
+        powers: P1 to P6 ordered list of 6 powers in W
+
+    :return: B02 parameters
+
+    """
+
+    _sort_order = ('payload', 'contrato1')
+
+    def __init__(self, payload, drop_empty=False):
+        powers = payload.get('powers')
+        act_date_param = payload.get('activation_date')
+
+        activation_date = datetimetoprime(act_date_param)
+
+        self.payload = XmlField(
+            'B02', attributes={
+                'ActDate': activation_date,
+            })
+
+        self.contrato1 = Contrato1({'powers': powers})
+        self.contrato1.feed({'powers': powers})
+
+        super(B02Payload, self).__init__('b02Payload', 'payload', drop_empty=drop_empty)
+
+
 class B03:
     """
     The class used to instance B03 order.
@@ -27,7 +99,7 @@ class B03:
             generic_values.get('id_req'),
             generic_values.get('cnc'),
             generic_values.get('cnt'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
         self.order.cnc.cnt.feed({'payload': B03Payload(payload)})
         # Load generic order with values
@@ -193,7 +265,7 @@ class B04:
             generic_values.get('id_req'),
             generic_values.get('cnc'),
             generic_values.get('cnt'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
         self.order.cnc.cnt.feed({'payload': B04Payload(payload)})
         # Load generic order with values
@@ -270,9 +342,9 @@ class B04Payload(XmlModel):
         super(B04Payload, self).__init__('b04Payload', 'payload', drop_empty=drop_empty)
 
 
-class B07IpFtp:
+class B07Ip:
     """
-    The class used to instance B07 order. Only for IPftp parameter.
+    The class used to instance B07 order. Only for IPftp/IPNTP/IPstg parameters.
     :return: B07 order with parameters
     """
     def __init__(self, generic_values, payload):
@@ -281,18 +353,26 @@ class B07IpFtp:
             generic_values.get('id_pet'),
             generic_values.get('id_req'),
             generic_values.get('cnc'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
-        self.order.cnc.feed({'payload': B07IpFtpPayload(payload)})
+        self.order.cnc.feed({'payload': B07IpPayload(payload)})
 
 
-class B07IpFtpPayload(XmlModel):
+class B07IpPayload(XmlModel):
     def __init__(self, payload, drop_empty=False):
+        attr_name = None
+        if 'IPftp' in payload:
+            attr_name = 'IPftp'
+        elif 'IPNTP' in payload:
+            attr_name = 'IPNTP'
+        elif 'IPstg' in payload:
+            attr_name = 'IPstg'
+
         self.payload = XmlField(
             'B07', attributes={
-                'IPftp': payload.get('IPftp'),
+                attr_name: payload.get(attr_name),
             })
-        super(B07IpFtpPayload, self).__init__('b07Payload', 'payload', drop_empty=drop_empty)
+        super(B07IpPayload, self).__init__('b07Payload', 'payload', drop_empty=drop_empty)
 
 
 class B07:
@@ -306,15 +386,20 @@ class B07:
             generic_values.get('id_pet'),
             generic_values.get('id_req'),
             generic_values.get('cnc'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
         b07 = B07Payload(payload)
-        tasks = payload.get("tasks")
+        tasks = payload.get("tasks", [])
         for task in tasks:
+            tppros = task.pop("TpPro")
             task_xml = B07Task(task)
-            tppros = task.get("task_data")
             for tppro in tppros:
+                attrs = tppro.pop('TpAttr')
                 tppro_xml = B07TpPro(tppro)
+
+                tpattr_xml = B07TpAttr(attrs)
+                tppro_xml.tpattr.append(tpattr_xml)
+
                 task_xml.tppro.append(tppro_xml)
             b07.tasks.append(task_xml)
         self.order.cnc.feed({'payload': b07})
@@ -326,7 +411,7 @@ class B07Payload(XmlModel):
     """
     def __init__(self, payload, drop_empty=False):
         # Discard empty strings and values and compose field
-        attributes = {k: v for k, v in payload.items() if v is not None and v != "" and k != "tasks"}
+        attributes = {k: v for k, v in payload.items() if v is not None and k != "tasks"}
         self.payload = XmlField('B07', attributes=attributes)
         self.tasks = []
         super(B07Payload, self).__init__('b07Payload', 'payload', drop_empty=drop_empty)
@@ -334,7 +419,7 @@ class B07Payload(XmlModel):
 
 class B07Task(XmlModel):
     def __init__(self, task, drop_empty=False):
-        attributes = {k: v for k, v in task.items() if v is not None and v != "" and k != "task_data"}
+        attributes = {k: v for k, v in task.items() if v is not None and k != "task_data"}
         self.task = XmlField('TP', attributes=attributes)
         self.tppro = []
         super(B07Task, self).__init__('TP', 'task', drop_empty=drop_empty)
@@ -342,11 +427,20 @@ class B07Task(XmlModel):
 
 class B07TpPro(XmlModel):
     def __init__(self, task, drop_empty=False):
-        attributes = {k: v for k, v in task.items() if v is not None and v != ""}
+        attributes = {k: v for k, v in task.items() if v is not None}
         self.tppro = XmlField('TpPro', attributes=attributes)
-        self.tpattr = XmlField('TpAttr')
+        self.tpattr = []
         super(B07TpPro, self).__init__('TpPro', 'tppro', drop_empty=drop_empty)
 
+
+class B07TpAttr(XmlModel):
+    def __init__(self, values, drop_empty=False):
+        self.tpattr = XmlField('TpAttr')
+        for k, v in values.items():
+            setattr(self, k, XmlField(k, value=v, parent='TpAttr'))
+
+        super(B07TpAttr, self).__init__('TpAttr', 'tpattr', drop_empty=drop_empty)
+
 class B09:
     """
     The class used to instance B09 order.
@@ -360,7 +454,7 @@ class B09:
             generic_values.get('id_req'),
             generic_values.get('cnc'),
             generic_values.get('cnt'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
         self.order.cnc.cnt.feed({'payload': B09Payload(payload)})
         # Load generic order with values
@@ -412,7 +506,7 @@ class B11:
             generic_values.get('id_pet'),
             generic_values.get('id_req'),
             generic_values.get('cnc'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
         self.order.cnc.feed({'payload': B11Payload(payload)})
         # Load generic order with values
@@ -460,7 +554,7 @@ class B12:
             generic_values.get('id_req'),
             generic_values.get('cnc'),
             generic_values.get('cnt'),
-            generic_values.get('version', DEFAULT_DC_STG_VERSION),
+            generic_values.get('version', '3.1.c'),
         )
         self.order.cnc.cnt.feed({'payload': B12Payload(payload)})
         # Load generic order with values
@@ -567,6 +661,10 @@ class Order(object):
         :return: Order formatted in XML
         """
         order_type_class = {
+            'B02': {
+                'class': B02,
+                'args': [generic_values, payload]
+            },
             'B03': {
                 'class': B03,
                 'args': [generic_values, payload]
@@ -575,16 +673,16 @@ class Order(object):
                 'class': B04,
                 'args': [generic_values, payload]
             },
-            'B09': {
-                'class': B09,
-                'args': [generic_values, payload]
-            },
             'B07': {
                 'class': B07,
                 'args': [generic_values, payload]
             },
-            'B07_ipftp': {
-                'class': B07IpFtp,
+            'B07_ip': {
+                'class': B07Ip,
+                'args': [generic_values, payload]
+            },
+            'B09': {
+                'class': B09,
                 'args': [generic_values, payload]
             },
             'B11': {
